@@ -1,34 +1,35 @@
+from __future__ import annotations
+
 import pickle
+from unittest.mock import patch
 
 import numpy as np
 import pytest
 
 import xarray as xr
 import xarray.ufuncs as xu
-
-from . import assert_array_equal
-from . import assert_identical as assert_identical_
-from . import assert_no_warnings, mock
+from xarray.tests import assert_allclose, assert_array_equal, mock, requires_dask
+from xarray.tests import assert_identical as assert_identical_
 
 
 def assert_identical(a, b):
     assert type(a) is type(b) or float(a) == float(b)
-    if isinstance(a, (xr.DataArray, xr.Dataset, xr.Variable)):
+    if isinstance(a, xr.DataArray | xr.Dataset | xr.Variable):
         assert_identical_(a, b)
     else:
         assert_array_equal(a, b)
 
 
-def test_unary():
-    args = [
-        0,
-        np.zeros(2),
+@pytest.mark.parametrize(
+    "a",
+    [
         xr.Variable(["x"], [0, 0]),
         xr.DataArray([0, 0], dims="x"),
         xr.Dataset({"y": ("x", [0, 0])}),
-    ]
-    for a in args:
-        assert_identical(a + 1, np.cos(a))
+    ],
+)
+def test_unary(a):
+    assert_allclose(a + 1, np.cos(a))
 
 
 def test_binary():
@@ -61,6 +62,19 @@ def test_binary_out():
         assert_identical(actual_exponent, arg)
 
 
+def test_binary_coord_attrs():
+    t = xr.Variable("t", np.arange(2, 4), attrs={"units": "s"})
+    x = xr.DataArray(t.values**2, coords={"t": t}, attrs={"units": "s^2"})
+    y = xr.DataArray(t.values**3, coords={"t": t}, attrs={"units": "s^3"})
+    z1 = xr.apply_ufunc(np.add, x, y, keep_attrs=True)
+    assert z1.coords["t"].attrs == {"units": "s"}
+    z2 = xr.apply_ufunc(np.add, x, y, keep_attrs=False)
+    assert z2.coords["t"].attrs == {}
+    # Check also that input array's coordinate attributes weren't affected
+    assert t.attrs == {"units": "s"}
+    assert x.coords["t"].attrs == {"units": "s"}
+
+
 def test_groupby():
     ds = xr.Dataset({"a": ("x", [0, 0, 0])}, {"c": ("x", [0, 0, 1])})
     ds_grouped = ds.groupby("c")
@@ -160,50 +174,106 @@ def test_gufuncs():
         xarray_obj.__array_ufunc__(fake_gufunc, "__call__", xarray_obj)
 
 
-def test_xarray_ufuncs_deprecation():
-    with pytest.warns(FutureWarning, match="xarray.ufuncs"):
-        xu.cos(xr.DataArray([0, 1]))
+class DuckArray(np.ndarray):
+    # Minimal subclassed duck array with its own self-contained namespace,
+    # which implements a few ufuncs
+    def __new__(cls, array):
+        obj = np.asarray(array).view(cls)
+        return obj
 
-    with assert_no_warnings():
-        xu.angle(xr.DataArray([0, 1]))
+    def __array_namespace__(self):
+        return DuckArray
+
+    @staticmethod
+    def sin(x):
+        return np.sin(x)
+
+    @staticmethod
+    def add(x, y):
+        return x + y
 
 
-@pytest.mark.filterwarnings("ignore::RuntimeWarning")
-@pytest.mark.parametrize(
-    "name",
-    [
-        name
-        for name in dir(xu)
-        if (
-            not name.startswith("_")
-            and hasattr(np, name)
-            and name not in ["print_function", "absolute_import", "division"]
-        )
-    ],
-)
-def test_numpy_ufuncs(name, request):
-    x = xr.DataArray([1, 1])
-
-    np_func = getattr(np, name)
-    if hasattr(np_func, "nin") and np_func.nin == 2:
-        args = (x, x)
-    else:
-        args = (x,)
-
-    y = np_func(*args)
-
-    if name in ["angle", "iscomplex"]:
-        # these functions need to be handled with __array_function__ protocol
-        assert isinstance(y, np.ndarray)
-    elif name in ["frexp"]:
-        # np.frexp returns a tuple
-        assert not isinstance(y, xr.DataArray)
-    else:
-        assert isinstance(y, xr.DataArray)
+class DuckArray2(DuckArray):
+    def __array_namespace__(self):
+        return DuckArray2
 
 
-@pytest.mark.filterwarnings("ignore:xarray.ufuncs")
-def test_xarray_ufuncs_pickle():
-    a = 1.0
-    cos_pickled = pickle.loads(pickle.dumps(xu.cos))
-    assert_identical(cos_pickled(a), xu.cos(a))
+class TestXarrayUfuncs:
+    @pytest.fixture(autouse=True)
+    def setUp(self):
+        self.x = xr.DataArray([1, 2, 3])
+        self.xd = xr.DataArray(DuckArray([1, 2, 3]))
+        self.xd2 = xr.DataArray(DuckArray2([1, 2, 3]))
+        self.xt = xr.DataArray(np.datetime64("2021-01-01", "ns"))
+
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
+    @pytest.mark.parametrize("name", xu.__all__)
+    def test_ufuncs(self, name, request):
+        xu_func = getattr(xu, name)
+        np_func = getattr(np, name, None)
+        if np_func is None and np.lib.NumpyVersion(np.__version__) < "2.0.0":
+            pytest.skip(f"Ufunc {name} is not available in numpy {np.__version__}.")
+
+        if name == "isnat":
+            args = (self.xt,)
+        elif hasattr(np_func, "nin") and np_func.nin == 2:
+            args = (self.x, self.x)
+        else:
+            args = (self.x,)
+
+        expected = np_func(*args)
+        actual = xu_func(*args)
+
+        if name in ["angle", "iscomplex"]:
+            np.testing.assert_equal(expected, actual.values)
+        else:
+            assert_identical(actual, expected)
+
+    def test_ufunc_pickle(self):
+        a = 1.0
+        cos_pickled = pickle.loads(pickle.dumps(xu.cos))
+        assert_identical(cos_pickled(a), xu.cos(a))
+
+    def test_ufunc_scalar(self):
+        actual = xu.sin(1)
+        assert isinstance(actual, float)
+
+    def test_ufunc_duck_array_dataarray(self):
+        actual = xu.sin(self.xd)
+        assert isinstance(actual.data, DuckArray)
+
+    def test_ufunc_duck_array_variable(self):
+        actual = xu.sin(self.xd.variable)
+        assert isinstance(actual.data, DuckArray)
+
+    def test_ufunc_duck_array_dataset(self):
+        ds = xr.Dataset({"a": self.xd})
+        actual = xu.sin(ds)
+        assert isinstance(actual.a.data, DuckArray)
+
+    @requires_dask
+    def test_ufunc_duck_dask(self):
+        import dask.array as da
+
+        x = xr.DataArray(da.from_array(DuckArray(np.array([1, 2, 3]))))
+        actual = xu.sin(x)
+        assert isinstance(actual.data._meta, DuckArray)
+
+    @requires_dask
+    @pytest.mark.xfail(reason="dask ufuncs currently dispatch to numpy")
+    def test_ufunc_duck_dask_no_array_ufunc(self):
+        import dask.array as da
+
+        # dask ufuncs currently only preserve duck arrays that implement __array_ufunc__
+        with patch.object(DuckArray, "__array_ufunc__", new=None, create=True):
+            x = xr.DataArray(da.from_array(DuckArray(np.array([1, 2, 3]))))
+            actual = xu.sin(x)
+            assert isinstance(actual.data._meta, DuckArray)
+
+    def test_ufunc_mixed_arrays_compatible(self):
+        actual = xu.add(self.xd, self.x)
+        assert isinstance(actual.data, DuckArray)
+
+    def test_ufunc_mixed_arrays_incompatible(self):
+        with pytest.raises(ValueError, match=r"Mixed array types"):
+            xu.add(self.xd, self.xd2)
